#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Classes of InsightLogger
"""

import numpy as np
from .utils import get_next_id
from .utils import ingraph
from .utils import parse_to_list
from .utils import parse_keywords
from .utils import parse_citations
from .utils import write_insight_log
from .utils import read_insight_log
from .utils import remove_insight_log


class InsightLogger(object):
    """
    Base InsightLogger class
    """

    def __init__(
        self,
        id=None,
        note=None,
        keywords=None,
        citations=None,
        relationships=None,
        save=False,
        load=False,
        **kwargs
    ):
        """[summary]

        Args:
            id ([type], optional): [description]. Defaults to None.
            note ([type], optional): [description]. Defaults to None.
            keywords ([type], optional): [description]. Defaults to None.
            citations ([type], optional): [description]. Defaults to None.
            relationships ([type], optional): [description]. Defaults to None.
            save (bool, optional): [description]. Defaults to False.
            load (bool, optional): [description]. Defaults to False.
        """

        if load:
            self.load(id)
        else:
            self.id = get_next_id()
            self.note = str(note) if note else note
            self.keywords = parse_keywords(keywords) if keywords else keywords
            self.citations = (
                parse_citations(citations, self.note) if citations else citations
            )
            self.relationships = relationships or []

        if save:
            self.save()

    def fields_to_dict(self):
        return {
            "id": self.id,
            "note": self.note,
            "keywords": self.keywords,
            "citations": self.citations,
            "relationships": self.relationships,
        }

    def save(self):
        """[summary]"""
        write_insight_log(self.fields_to_dict())

    def load(self, id: str):
        """[summary]

        Args:
            id (str): [description]
        """
        fields = read_insight_log(id)
        self.__init__(**fields)

    def empty(self, id: str):
        """[summary]

        Args:
            id (str): [description]
        """
        remove_insight_log(id)

    def add_citations(self, input):
        """[summary]

        Args:
            input ([type]): [description]
        """
        citations_to_add = parse_to_list(input)
        for citation in citations_to_add:
            if citation not in self.citations:
                self.citations.append(citation)

    def add_keywords(self, input):
        """[summary]

        Args:
            input ([type]): [description]
        """
        keywords_to_add = parse_citations(input)
        for keyword in keywords_to_add:
            if keyword not in self.keywords:
                self.keywords.append(keyword)

    def add_relationship(
        self, action: str, action_subjects, action_objects, directional=False
    ):
        """[summary]

        Args:
            action (str): [description]
            action_subjects ([type]): [description]
            action_objects ([type]): [description]
            directional (bool, optional): [description]. Defaults to False.
        """
        action_subjects_list = parse_to_list(action_subjects)
        action_objects_list = parse_to_list(action_objects)

        for action_subject in action_subjects_list:
            for action_object in action_objects_list:
                if ingraph(
                    graph=self.relationships,
                    action=action,
                    action_subject=action_subject,
                    action_object=action_object,
                ):
                    self.relationships.append([action, action_subject, action_object])
                if not directional and ingraph(
                    graph=self.relationships,
                    action=action,
                    action_subject=action_object,
                    action_object=action_subject,
                ):
                    self.relationships.append([action, action_object, action_subject])



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Classes of PaperAssistant
"""

from abc import abstractclassmethod
import numpy as np


class PaperAssistant(object):
    """
    Base PaperAssistant class
    """

    def __init__(self, name=None, seed=0, **kwargs):
        self.name = name
        self.seed = seed
        np.random.seed(seed)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Classes of TopicCurator
"""

import numpy as np


class TopicCurator(object):
    """
    Base TopicCurator class
    """

    def __init__(self, name=None, seed=0, **kwargs):
        self.name = name
        self.seed = seed
        np.random.seed(seed)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
utils functions related to paper
"""


def inlog(paper_id: str):
    """[summary]

    Args:
        paper_id (str): [description]

    Returns:
        [type]: [description]
    """
    return False


def write_paper_log(fields: dict):
    """[summary]

    Args:
        fields (dict): [description]
    """
    pass


def read_paper_log(paper_id: str):
    """[summary]

    Args:
        paper_id (str): [description]

    Returns:
        [type]: [description]
    """
    return None


def remove_paper_log(paper_id: str):
    """[summary]

    Args:
        paper_id (str): [description]
    """
    pass


def parse_authors(authors_string: str):
    """[summary]

    Args:
        authors_string (str): [description]
    """
    return []



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Classes of PaperLogger
"""

import numpy as np
from .utils import parse_authors
from .utils import write_paper_log
from .utils import read_paper_log
from .utils import remove_paper_log


class PaperLogger(object):
    """
    Base PaperLogger class
    """

    def __init__(
        self,
        id=None,
        title=None,
        authors=None,
        venue=None,
        year=None,
        papertype="research",
        bib=None,
        abstract=None,
        save=False,
        load=False,
        **kwargs
    ):
        """[summary]

        Args:
            id ([type], optional): [description]. Defaults to None.
            title ([type], optional): [description]. Defaults to None.
            authors ([type], optional): [description]. Defaults to None.
            venue ([type], optional): [description]. Defaults to None.
            year ([type], optional): [description]. Defaults to None.
            papertype (str, optional): [description]. Defaults to "research".
            bib ([type], optional): [description]. Defaults to None.
            abstract ([type], optional): [description]. Defaults to None.
            save (bool, optional): [description]. Defaults to False.
            load (bool, optional): [description]. Defaults to False.
        """
        self.loaded = load
        
        #TODO needs to figure out the relationship between save and load
        if load:
            self.load(id)
        else:
            self.id = id
            self.title = str(title) if title else title
            self.authors = str(authors) if authors else authors
            self.authors_list = parse_authors(authors)
            self.venue = str(venue) if venue else venue
            self.year = int(year) if year else year
            self.papertype = str(papertype) if papertype else papertype
            self.bib = str(bib) if bib else bib
            self.abstract = str(abstract) if abstract else abstract
            self.other_variables = kwargs

        if save:
            self.save()

    def fields_to_dict(self):
        return {
            "id": self.id,
            "title": self.title,
            "authors": self.authors,
            "authors_list": self.authors_list,
            "venue": self.venue,
            "year": self.year,
            "papertype": self.papertype,
            "bib": self.bib,
            "abstract": self.abstract,
        }

        
    def save(self):
        """[summary]"""
        fields_dict = self.fields_to_dict()
        other_variables = self.other_variables
        write_paper_log({**fields_dict, **other_variables})

    def load(self, id: str):
        """[summary]

        Args:
            id (str): [description]
        """
        fields = read_paper_log(id)
        self.__init__(**fields)

    def empty(self, id: str):
        """[summary]

        Args:
            id (str): [description]
        """
        remove_paper_log(id)

    def update_id(self, id:str):
        """[summary]"""
        #TODO needs fixing
        
        fields_dict = self.fields_to_dict()
        fields_dict['id']
        other_variables = self.other_variables
        write_paper_log({**fields_dict, **other_variables})


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
utils functions related to insight
"""


def inlog(insight_id: str):
    """[summary]

    Args:
        insight_id (str): [description]

    Returns:
        [type]: [description]
    """
    return False


def ingraph(graph, action: str, action_subject: str, action_object: str):
    """[summary]

    Args:
        graph ([type]): [description]
        action (str): [description]
        action_subject (str): [description]
        action_object (str): [description]

    Returns:
        [type]: [description]
    """
    return False


def get_next_id():
    """[summary]

    Returns:
        [type]: [description]
    """
    return 0


def write_insight_log(fields: dict):
    """[summary]

    Args:
        fields (dict): [description]
    """
    pass


def read_insight_log(insight_id: str):
    """[summary]

    Args:
        insight_id (str): [description]

    Returns:
        [type]: [description]
    """
    return None


def remove_insight_log(insight_id: str):
    """[summary]

    Args:
        insight_id (str): [description]
    """
    pass


def parse_to_list(input):
    """[summary]

    Args:
        input ([type]): [description]

    Returns:
        [type]: [description]
    """
    return []


def parse_keywords(input):
    """[summary]

    Args:
        input ([type]): [description]

    Returns:
        [type]: [description]
    """
    return parse_to_list(input)


def parse_citations(input, note: str):
    """[summary]

    Args:
        input ([type]): [description]

    Returns:
        [type]: [description]
    """
    return []
